
/**********************************************************************************
 Persistence of Vision Ray Tracer Scene Description File
 File name   : Eval_pigmentTest.pov
 Version     : 3.6
 Description : Use of eval_pigment() to place objects in combination with trace()
               on a height_field or an isosurface
               with thanks to Norbert Kern's "Warm-up" (1st place IRTC May-June 2001)
               
 Date        : March 2005
 Author      : Thomas A. M. de Groot
 E-mail      : t.degroot@inter.nl.net
 
 Copyright (C) 2005. All rights reserved.
**********************************************************************************/

#version 3.6;

#include "colors.inc"
#include "functions.inc"
#include "rand.inc"

global_settings {
  assumed_gamma 1.0
}

default {
  texture {
    pigment { rgb <1,0,0> }
  }
}
// ----------------------------------------

camera {
  location  <0.0, 1, -3.0>
  direction 1.5*z
  right     x*image_width/image_height
  look_at   <0.0, 0.0,  0.0>
}

sky_sphere {
  pigment {
    gradient y
    color_map {
      [0.0 rgb <0.6,0.7,1.0>]
      [0.7 rgb <0.0,0.1,0.8>]
    }
  }
}

light_source {
  <0, 0, 0>            // light's position (translated below)
  color rgb <1, 1, 1>  // light's color
  translate <-30, 30, -30>
}

// ----------------------------------------

plane {
  y, -1
  pigment { color rgb <0.7,0.5,0.3> }
}

#declare SpottedMap = 
pigment {
  image_map {
    png "Spotted.png"
  } 
  rotate x*90 
  translate <-0.5,-0.5,-0.5> 
  scale <2,0.1,2>
} 

#declare HF =
height_field {
  png "Spotted.png" 
  smooth         
  translate <-0.5,-0.5,-0.5>
  scale <2,0.1,2>
  texture {
    pigment {
      SpottedMap
    }
  }
  normal {
    granite
  }
  finish {
    phong .5 
    crand .5 
    ambient .05 
    diffuse .7
  }
}

object {HF}

#declare Rod =
cylinder { <0, 0, 0>, <0, 0.2, 0>, 0.005 }


//Little macro from "warm-up" by Norbert Kern (1st place, IRTC May-June 2001)
#macro Lum(col) col.x*.3333+col.y*.3333+col.z*.3333 #end

// -------- Placing objects -------------
#declare Seed=seed(2);
#declare Seed2=seed(1);

#declare Spacing = 0.1;
#declare PosX = -2;

#while (PosX < 2)
  #declare PosZ = -2;
  #while (PosZ < 2)
    // trace function
    #declare Norm = <0, 0, 0>;
    #declare Start = <PosX+(rand(Seed)-0.5)*Spacing, 2, PosZ+(rand(Seed)-0.5)*Spacing>;
    #declare Pos = trace (
                  HF,     	 // surface to test
                  Start,     // starting point
                  -y,        // direction
                  Norm );    // normal

    #if (Norm.x != 0 | Norm.y != 0 | Norm.z != 0)   // if intersection is found, normal differs from 0
    
      #if ((vdot(Norm, y)>0.3) & (Pos.y > -1.0))  	// criteria for placing trees: not too steep and not too high
      
        // selection of texture according to eval_pigment of the image_map (see also Norbert Kern)
        #if (Lum(eval_pigment(SpottedMap,<PosX,0,PosZ>))<0.5)
          object {
            Rod
            translate Pos
          }
        #else
          object {
            Rod
            texture {
              pigment {
                rgb <0,0,1> 
              }
            }
            translate Pos
          }
       #end //end of eval_pigment
        
      #end //end of vdot() & Pos.y
      
    #end //end of Norm

    #declare PosZ=PosZ+Spacing;
  #end //end of while Z

  #declare PosX=PosX+Spacing;
#end //end of while X

